// Copyright 2015 Aaron Jacobs. All Rights Reserved.
// Author: aaronjjacobs@gmail.com (Aaron Jacobs)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Package oglemock provides a mocking framework for unit tests.
//
// Among its features are the following:
//
//  *  An extensive and extensible set of matchers for expressing call
//     expectations (provided by the oglematchers package).
//
//  *  Style and semantics similar to Google Mock and Google JS Test.
//
//  *  Easy integration with the ogletest unit testing framework.
//
// See https://github.com/smartystreets/assertions/internal/oglemock for more information.
package oglemock
